@@ -41,6 +41,7 @@ from lsprotocol.types import (
 )
 from pygls.workspace import TextDocument
 
+from .constants import MAX_CONCURRENT_DEBOUNCE_CALLS
 from .initialization_options import HoverDisableOptions, InitializationOptions
 from .type_map import get_lsp_completion_type, get_lsp_symbol_type
 
@@ -53,13 +54,15 @@ else:
 P = ParamSpec("P")
 
 
+_debounce_semaphore = threading.Semaphore(MAX_CONCURRENT_DEBOUNCE_CALLS)
+
+
 def debounce(
-    interval_s: int, keyed_by: Optional[str] = None
+    interval_s: float, keyed_by: Optional[str] = None
 ) -> Callable[[Callable[P, None]], Callable[P, None]]:
     """Debounce calls to this function until interval_s seconds have passed.
 
-    Decorator copied from https://github.com/python-lsp/python-lsp-
-    server
+    Decorator adapted from https://github.com/python-lsp/python-lsp-server
     """
 
     def wrapper(func: Callable[P, None]) -> Callable[P, None]:
@@ -68,20 +71,25 @@ def debounce(
 
         @functools.wraps(func)
         def debounced(*args: P.args, **kwargs: P.kwargs) -> None:
+            _debounce_semaphore.acquire()
+
             sig = inspect.signature(func)
             call_args = sig.bind(*args, **kwargs)
             key = call_args.arguments[keyed_by] if keyed_by else None
 
             def run() -> None:
-                with lock:
-                    del timers[key]
-                return func(*args, **kwargs)
+                try:
+                    with lock:
+                        del timers[key]
+                    func(*args, **kwargs)
+                finally:
+                    _debounce_semaphore.release()
 
             with lock:
                 old_timer = timers.get(key)
                 if old_timer:
                     old_timer.cancel()
-
+                    _debounce_semaphore.release()
                 timer = threading.Timer(interval_s, run)
                 timers[key] = timer
                 timer.start()
@@ -148,22 +156,26 @@ def lsp_range(name: Name) -> Optional[Range]:
     )
 
 
-def lsp_location(name: Name) -> Optional[Location]:
+def lsp_location(name: Name, uri: Optional[str] = None) -> Optional[Location]:
     """Get LSP location from Jedi definition."""
-    module_path = name.module_path
-    if module_path is None:
-        return None
+    if uri is None:
+        module_path = name.module_path
+        if module_path is None:
+            return None
+        uri = module_path.as_uri()
 
     lsp = lsp_range(name)
     if lsp is None:
         return None
 
-    return Location(uri=module_path.as_uri(), range=lsp)
+    return Location(uri=uri, range=lsp)
 
 
-def lsp_symbol_information(name: Name) -> Optional[SymbolInformation]:
+def lsp_symbol_information(
+    name: Name, uri: Optional[str] = None
+) -> Optional[SymbolInformation]:
     """Get LSP SymbolInformation from Jedi definition."""
-    location = lsp_location(name)
+    location = lsp_location(name, uri)
     if location is None:
         return None
 
@@ -406,13 +418,17 @@ def complete_sort_name(name: Completion, append_text: str) -> str:
     return "v" + append_text
 
 
-def clean_completion_name(name: str, char_before_cursor: str) -> str:
+def clean_completion_name(
+    name: str, char_before_cursor: str, char_after_cursor: str
+) -> str:
     """Clean the completion name, stripping bad surroundings.
 
     Currently, removes surrounding " and '.
     """
     if char_before_cursor in {"'", '"'}:
-        return name.lstrip(char_before_cursor)
+        name = name.lstrip(char_before_cursor)
+    if char_after_cursor in {"'", '"'}:
+        name = name.rstrip(char_after_cursor)
     return name
 
 
@@ -484,6 +500,7 @@ def clear_completions_cache() -> None:
 def lsp_completion_item(
     completion: Completion,
     char_before_cursor: str,
+    char_after_cursor: str,
     enable_snippets: bool,
     resolve_eagerly: bool,
     markup_kind: MarkupKind,
@@ -491,11 +508,13 @@ def lsp_completion_item(
 ) -> CompletionItem:
     """Using a Jedi completion, obtain a jedi completion item."""
     completion_name = completion.name
-    name_clean = clean_completion_name(completion_name, char_before_cursor)
+    name_clean = clean_completion_name(
+        completion_name, char_before_cursor, char_after_cursor
+    )
     lsp_type = get_lsp_completion_type(completion.type)
     completion_item = CompletionItem(
-        label=completion_name,
-        filter_text=completion_name,
+        label=name_clean,
+        filter_text=name_clean,
         kind=lsp_type,
         sort_text=complete_sort_name(completion, sort_append_text),
         insert_text=name_clean,
@@ -521,7 +540,7 @@ def lsp_completion_item(
         snippet_signature = get_snippet_signature(signatures[0])
     except Exception:
         return completion_item
-    new_text = completion_name + snippet_signature
+    new_text = name_clean + snippet_signature
     completion_item.insert_text = new_text
     completion_item.insert_text_format = InsertTextFormat.Snippet
     return completion_item
